DROP VIEW IF EXISTS reviewer_item_actions_v;
DROP VIEW IF EXISTS suser_v;

CREATE VIEW reviewer_item_actions_v AS
  SELECT 
    review_instance.id,
    review_instance.status review_status,
    review_instance.start_date,
    review_instance.end_date,
    review_instance.rev_name,
    review_instance.review_type,
    review_instance.unique_review_def_id,
    all_reviewers.reviewer_id,
    all_reviewers.display_name reviewer_name,
    all_reviewers.item_id,
    all_reviewers.item_name,
    all_reviewers.item_type,
    all_reviewers.reviewer_comment,
    all_reviewers.action,
    all_reviewers.reviewer_type,
    all_reviewers.review_order
  FROM (SELECT 
      rev_inst.id,
      rev_inst.status,
      rev_inst.start_date,
      rev_inst.end_date,
      COALESCE(staged_defn.name, defn.name) rev_name,
      defn.review_type,
      defn.unique_review_def_id
    FROM review_instance rev_inst
      JOIN review_definition defn ON defn.id = rev_inst.review_definition
      LEFT JOIN review_definition staged_defn ON staged_defn.id = rev_inst.staged_review_definition_id
    WHERE rev_inst.status NOT LIKE 'PREVIEW%') review_instance
    JOIN (SELECT 
        item.review_instance_id,
        COALESCE(ria.acted_by_user_unique_id, ria.reassigned_by_user_unique_id) reviewer_id,
        COALESCE(usr.display_name, usr2.display_name) display_name,
        ria.action,
        ria.review_order,
        item.item_type,
        item.id item_id,
        COALESCE(perm.name, acct.name, trole.name, brole.name) item_name,
        ria.reviewer_type,
        ria.reviewer_comment
      FROM review_item item
        JOIN review_item_action ria ON ria.review_item_id = item.id AND ria.action IN ('KEEP', 'REMOVE', 'MODIFY_ACCESS', 'REASSIGNED')
        LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = true
        LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = true
        LEFT JOIN role_policy trole ON trole.id = item.role_id AND trole.state = 'ACTIVE'
        LEFT JOIN business_role brole ON brole.id = item.business_role_id AND brole.approval_state != 'ARCHIVE'
        LEFT JOIN suser usr ON usr.unique_user_id = ria.acted_by_user_unique_id AND usr.effective = true
        LEFT JOIN suser usr2 ON usr2.unique_user_id = ria.reassigned_by_user_unique_id AND usr2.effective = true
      
      UNION ALL
      
      SELECT
        auditor.review_instance_id,
        rui.unique_user_id auditor_unique_id,
        usr.display_name,
          CASE
            WHEN rev_inst.status = 'CERTIFIED' THEN 'ACCEPTED'
            ELSE 'REJECTED'
          END AS rev_action,
        auditor.review_order,
        NULL item_type,
        NULL item_id,
        NULL item_name,
        auditor.type,
        rev_inst.auditor_comment
      FROM reviewer_instance auditor
        JOIN reviewer_user_instance rui ON auditor.id = rui.reviewer_instance_id
        JOIN review_instance rev_inst ON auditor.review_instance_id = rev_inst.id AND rev_inst.status IN ('CERTIFIED', 'NOT_CERTIFIED')
        JOIN suser usr ON usr.unique_user_id = rui.unique_user_id AND usr.effective = true
      WHERE auditor.type = 'AUDITOR') all_reviewers ON all_reviewers.review_instance_id = review_instance.id AND all_reviewers.reviewer_id IS NOT NULL;

CREATE VIEW suser_v AS 
SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    suser.first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    suser.job_title,
    suser.last_name,
    suser.location,
    suser.merged,
    suser.middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    suser.unique_user_id,
    suser.user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    suser.workforce_id,
    suser_email.email,
    suser.risk
FROM suser
    LEFT JOIN 
        (SELECT
            suser.id suser,
            suser.display_name,
            string_agg(suser_email.email, ' | ' order by suser_email.email) As email
        FROM suser
        JOIN suser_email ON suser.id = suser_email.suser AND suser.effective = '1' AND suser.deleted = '0'   
        GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
WHERE suser.deleted = '0' AND suser.effective = '1';

ALTER TABLE reviewer_item_actions_v
  OWNER TO igops;
GRANT ALL ON TABLE reviewer_item_actions_v TO igops;
GRANT SELECT ON TABLE reviewer_item_actions_v TO ig_report_role;

ALTER TABLE suser_v
  OWNER TO igops;
GRANT ALL ON TABLE suser_v TO igops;
GRANT SELECT ON TABLE suser_v TO ig_report_role;